// Programmed using the knockoff Pro Micro boards - use Arduino/Genuino Micro as board type
// Watch out - your board's name might change and you'll have to seelct it again in Tools/Port 
// after you program it the first time

// This program uses a resitor ladder and 4 analog inputs to provide 
// Generative music a "patch"able environment - think of it
// as a modular sythesizer sequencer that sends out MIDI notes
// changing the algorithm below will change the music it makes!

// Libraries 
// You need to install both libraries - search for them under Tools/Manage Libraries, and then 
// search for the word in front of the .h
#include <elapsedMillis.h>
#include <MIDIUSB.h>

// You can change which pitch plays by changing these MIDI Note Numbers
// You need to have 16 numbers here. They don't all have to be different, but there
// have to be 16 total.
byte pitch[] = {43, 47, 48, 52, 55, 59, 60, 64, 67, 71, 72, 76, 79, 83, 84, 88};

// Parameters we'll use to control the system output
int A = 0;
int B = 0;
int C = 0;
int D = 0;
int AB = 0;
int AmB = 0;
int AC = 0;
int AmC = 0;
int AD = 0;
int AmD = 0;
int BC = 0;
int BmC = 0;
int BD = 0;
int BmD = 0;
int CD = 0;
int CmD = 0;

// MIDI Parameters
int voiceAchannel = 0;    // Send voice A to Midi Channel 1 (zero-indexed)
int voiceBchannel = 1;    // Send voice B to Midi Channel 2 (zero-indexed)
int voiceCchannel = 2;    // Send voice C to Midi Channel 3 (zero-indexed)
int CC = 100;             // send control change variables on CC 100

// Values for storing previously played and current notes, so we can turn things ON/OFF
int voiceAcurrP = 0;
int voiceAprevP = 0;
int voiceBcurrP = 0;
int voiceBprevP = 0;
int voiceCcurrP = 0;
int voiceCprevP = 0;

// State Variables to keep track of whether or not the note is currently active
bool Astate = 0;
bool Bstate = 0;
bool Cstate = 0;

// Timer variables
elapsedMillis tick = 0; // Fundamental clock unit
elapsedMillis CCtimer = 0; // Timer to limit the amount of CC data sent
elapsedMillis Atick = 0; // Fundamental clock unit
elapsedMillis Btick = 0; // Fundamental clock unit
elapsedMillis Ctick = 0; // Fundamental clock unit

void setup() {
  // put your setup code here, to run once:
  pinMode(A0,INPUT_PULLUP);  // Use an internal pullup resistor so 
  pinMode(A1,INPUT_PULLUP);  // that when nothing is connected it
  pinMode(A2,INPUT_PULLUP);  // reads a value close to maximum
  pinMode(A3,INPUT_PULLUP);
}
void loop() {
  // put your main code here, to run repeatedly:
  updateValues();
  voiceA();
  voiceB();
  voiceC();
  CClfo();
}

void updateValues(){
  A = analogRead(A0);
  B = analogRead(A1);
  C = analogRead(A2);
  D = analogRead(A3);
  AB = (A+B)/2;
  AmB = abs(A-B)/2;
  AC = (A+C)/2;
  AmC = abs(A-C)/2;
  AD = (A+D)/2;
  AmD = abs(A-D)/2;
  BC = (B+C)/2;
  BmC = abs(B-C)/2;
  BD = (B+D)/2;
  BmD = abs(B-D)/2;
  CD = (C+D)/2;
  CmD = abs(C-D)/2;
}

// Send notes for voice B
void voiceA(){
  // Define local variables
  int v1 = AB;
  int v2 = AmB;
  int v3 = AC;
  int v4 = AmC;

  if(Astate && (Atick > map(v2+v4,0,1023,200,400))){
    noteOff(voiceAchannel,voiceAprevP,0);
    Astate = 0;
  } else if(Atick>map(v3,0,1023,800,5000)){
    Atick = 0;
    int pitchVal = A/64;
    int velVal = map(v1, 0, 1023, 30, 120);
    noteOff(voiceAchannel,voiceAprevP,0);
    voiceAcurrP = pitch[pitchVal];
    noteOn(voiceAchannel, voiceAcurrP, velVal);
    Astate = 1;
    voiceAprevP = voiceAcurrP;
  }
}

// Send notes for voice B
void voiceB(){
  // Define local variables
  int v1 = BC;
  int v2 = BmC;
  int v3 = BD;
  int v4 = BmD;

  if(Bstate && (Btick > map(v2+v4,0,1023,200,400))){
    noteOff(voiceBchannel,voiceBprevP,0);
    Bstate = 0;
  } else if(Btick>map(v3,0,1023,800,5000)){
    Btick = 0;
    int pitchVal = B/64;
    int velVal = map(v1, 0, 1023, 30, 120);
    noteOff(voiceBchannel,voiceBprevP,0);
    voiceBcurrP = pitch[pitchVal];
    noteOn(voiceBchannel, voiceBcurrP, velVal);
    Bstate = 1;
    voiceBprevP = voiceBcurrP;
  }  
}

// Send notes for voice C
void voiceC(){
  // Define local variables
  int v1 = AD;
  int v2 = AmD;
  int v3 = CD;
  int v4 = CmD;

  if(Cstate && (Ctick > map(v2+v4,0,1023,200,400))){
    noteOff(voiceCchannel,voiceCprevP,0);
    Cstate = 0;
  } else if(Ctick>map(v3,0,1023,800,5000)){
    Ctick = 0;
    int pitchVal = C/64;
    int velVal = map(v1, 0, 1023, 30, 120);
    noteOff(voiceCchannel,voiceCprevP,0);
    voiceCcurrP = pitch[pitchVal];
    noteOn(voiceCchannel, voiceCcurrP, velVal);
    Cstate = 1;
    voiceCprevP = voiceCcurrP;
  }    
}


// Function to send LFO data via a CC message
// Include Rate (AB), Depth (CD), and offset(D)
void CClfo(){
  if(CCtimer>30){
   
    CCtimer = 0;
    int rate  = map(AB, 0, 1023, 0, 63);
    int depth = map(CD, 0, 1023, 0, 20);
    int offset = map(D, 0, 1023, 50, 90);
    
    // Calculate the new CC value
    int val = int(depth*cos(rate*float(tick)/1000))+offset;
    
    // Send it to all MIDI channels
    sendCC(voiceAchannel, CC, val);
//    sendCC(voiceBchannel, CC, val);
//    sendCC(voiceCchannel, CC, val);
  }
}

// A function to make it easier to send control change messages
void sendCC(byte channel, byte CC, byte value) {
  midiEventPacket_t CCdata = {0x0B, 0xB0 | channel, CC, value};
  MidiUSB.sendMIDI(CCdata);
  MidiUSB.flush();
}

// A function to make it easier to send Note On messages
void noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = {0x09, 0x90 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
  MidiUSB.flush();
}

// A function to make it easier to send Note Off messages
void noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = {0x08, 0x80 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
  MidiUSB.flush();
}
